/* 
   - The contents of this file are subject to the Mozilla Public
   - License Version 1.1 (the "License"); you may not use this file
   - except in compliance with the License. You may obtain a copy of
   - the License at http://www.mozilla.org/MPL/
   -
   - Software distributed under the License is distributed on an "AS
   - IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
   - implied. See the License for the specific language governing
   - rights and limitations under the License.
   -
   - The Original Code is this file as it was released on
   - March 20, 2000.
   -
   - The Initial Developer of the Original Code is Henk-Johan van Rantwijk.
   - Portions created by Henk-Johan van Rantwijk are Copyright (C) 2000
   - Henk-Johan van Rantwijk.  All Rights Reserved.
   -
   - Contributor(s):
   -  Henk-Johan van Rantwijk <bugs4hj@netscape.net> (Original Author)
   -
   - Alternatively, the contents of this file may be used under the
   - terms of the GNU General Public License Version 2 or later (the
   - "GPL"), in which case the provisions of the GPL are applicable
   - instead of those above.  If you wish to allow use of your
   - version of this file only under the terms of the GPL and not to
   - allow others to use your version of this file under the MPL,
   - indicate your decision by deleting the provisions above and
   - replace them with the notice and other provisions required by
   - the GPL.  If you do not delete the provisions above, a recipient
   - may use your version of this file under either the MPL or the
   - GPL.
   -
*/

var mzToolbox = null;
var gConfirmForID = -1;

// I am *so* going to group some of these listeners!
const gbToolboxVisibilityPrefListener =
{
  domains: [ "multizilla.integrated.toolbar",
             "multizilla.integrated.toolbar.hide-all-other-buttons" ],
  observe: function(subject, topic, aPrefName) {
    if (topic != "nsPref:changed")
      return;

    var prefValue = pref.getBoolPref(aPrefName);

    if (aPrefName == this.domains[0])
      mzToolbox.mzToolboxSetVisibility(prefValue);
    else if (aPrefName == this.domains[1])
      mzToolbox.refreshMultiZillaButtons();
  }
};
const gTabMinWidthPrefListener =
{
  domain: "multizilla.tabs.mintabwidth",
  observe: function(subject, topic, prefName) {
    if (topic != "nsPref:changed")
      return;

    var tabWidth = pref.getIntPref(prefName);
    gBrowser.setTabWidthTo("minwidth", tabWidth);
  }
};
const gTabWidthPrefListener =
{
  domain: "multizilla.tabs.tabwidth",
  observe: function(subject, topic, prefName) {
    if (topic != "nsPref:changed")
      return;

    var tabWidth = pref.getIntPref(prefName);
    gBrowser.setTabWidthTo("maxwidth", tabWidth);
  }
};
const gTabLEDstatePrefListener =
{
  domain: "multizilla.tabs.status-leds",
  observe: function(subject, topic, prefName) {
    if (topic != "nsPref:changed")
      return;

    var tabLEDstate = pref.getBoolPref(prefName);
    gBrowser.setTabLEDstateTo(tabLEDstate);
  }
};
const gTabLabelPrefListener =
{
  domain: "multizilla.tabs.title",
  observe: function(subject, topic, prefName) {
    if (topic != "nsPref:changed")
      return;

    var tabLabel = pref.getBoolPref(prefName);
    gBrowser.setTabLabelsTo(tabLabel);
  }
};
const gTabLabelCropPrefListener =
{
  domain: "multizilla.tabs.cropping",
  observe: function(subject, topic, prefName) {
    if (topic != "nsPref:changed")
      return;
    gBrowser.setLabelCropping();
  }
};
const gTabSwitchPrefListener =
{
  domain: "multizilla.tabs.ontop",
  observe: function(subject, topic, prefName) {
    if (topic != "nsPref:changed")
      return;

    var topTabs = pref.getBoolPref(prefName);
    if (topTabs)
      gBrowser.activateTopTabs()
    else gBrowser.activateBottomTabs();
  }
};
const gTabCanClosePrefListener =
{
  domain: "multizilla.tabs.iconclose",
  observe: function(subject, topic, prefName) {
    if (topic != "nsPref:changed")
      return;

    var tabCanClose = pref.getBoolPref(prefName);
    gBrowser.setTabCanClose(tabCanClose);
  }
};
const gTBButtonPrefListener =
{
  domain: "multizilla.showbutton",
  observe: function(subject, topic, prefName)   {
    if (topic != "nsPref:changed")
      return;
    var buttonName = prefName.substr(this.domain.length+1);
    var buttonId = buttonName + "-tbbutton";
    var button = document.getElementById(buttonId);
    var show = pref.getBoolPref(prefName);

    if (show) 
      button.removeAttribute("hidden");
    else button.setAttribute("hidden", "true");
  }
};
const gOrdinancePrefListener =
{
  domain: "multizilla.tabs.open-order",
  observe: function(subject, topic, prefName) {
    if (topic != "nsPref:changed")
      return;

    var ordinance = pref.getIntPref(prefName);
    gBrowser.mTabOrdinance = ordinance;
  }
};
const gtabFocusOrderPrefListener =
{
  domain: "multizilla.tabs.focus-order",
  observe: function(subject, topic, prefName) {
    if (topic != "nsPref:changed")
      return;

    var tabFocusOrder = pref.getIntPref(prefName);
    gBrowser.mTabFocus = tabFocusOrder;
  }
};
const gtabbarCloseButtonPrefListener =
{
  domain: "multizilla.tabbar.closebutton",
  observe: function(subject, topic, prefName) {
    if (topic != "nsPref:changed")
      return;

    var closeButton = pref.getBoolPref(prefName);
    gBrowser.mTabContainer.setAttribute("displayclose", closeButton ? "true" : "false");
  }
};
const gtabbarNewButtonPrefListener =
{
  domain: "multizilla.tabbar.newbutton",
  observe: function(subject, topic, prefName) {
    if (topic != "nsPref:changed")
      return;

    var newButton = pref.getBoolPref(prefName);
    gBrowser.mTabContainer.setAttribute("displaynew", newButton ? "true" : "false");
  }
};
const gTabSiteIconsPrefListener =
{
  domain: "browser.chrome.site_icons",
  observe: function(subject, topic, prefName) {
    if (topic != "nsPref:changed")
      return;

    gBrowser.updateDisplayTabIcons();
  }
};
const gTabFaviconsPrefListener =
{
  domain: "browser.chrome.favicons",
  observe: function(subject, topic, prefName) {
    if (topic != "nsPref:changed")
      return;

    gBrowser.updateDisplayTabIcons();
  }
};
const gTabAdvancedOpenTabPrefListener =
{
  domains: [ "multizilla.newtabfor.native-applications",
             "advanced.system.supportDDEExec" ],

  observe: function(subject, topic, aPrefName) {
    if (topic != "nsPref:changed")
      return;

    if (aPrefName == "multizilla.newtabfor.native-applications")
      pref.setBoolPref(this.domains[1], !pref.getBoolPref(aPrefName));
    else if (aPrefName == "advanced.system.supportDDEExec")
      pref.setBoolPref(this.domains[0], !pref.getBoolPref(aPrefName));
  }
};
const gTabSessionStartupPrefListener =
{
  domains: [ "browser.startup.page",
             "multizilla.browsersession.skip.navigator.startup",
             "multizilla.tabs.startgroup" ],

  observe: function(subject, topic, aPrefName) {
    if (topic != "nsPref:changed")
      return;

    var prefValue;

    if (aPrefName == "multizilla.browsersession.skip.navigator.startup") {
      prefValue = pref.getBoolPref(aPrefName);
      if (prefValue)
        pref.setIntPref(this.domains[0], 0);
    }
    else if (aPrefName == "browser.startup.page") {
      prefValue = pref.getIntPref(aPrefName);
      if (prefValue > 0)
        pref.setBoolPref(this.domains[1], false);
    }
    else if (aPrefName == "multizilla.tabs.startgroup") {
      prefValue = pref.getBoolPref(aPrefName);
      if (prefValue)
        pref.setBoolPref(this.domains[1], true);
    }
  }
};
const gTabMetaRefreshPrefListener =
{
  domain: "multizilla.tabs.meta-refresh",
  observe: function(subject, topic, prefName) {
    if (topic != "nsPref:changed")
      return;
    gBrowser.mMetaRefresh = pref.getIntPref(prefName);
  }
};
//////////////////////////////////////////////////////////////////////////
// Replacement for 'gPopupPrefListener' in navigator.js
//////////////////////////////////////////////////////////////////////////
const mzPopupPrefListener =
{
  domain: "dom.disable_open_during_load",
  observe: function(subject, topic, prefName)
  {        
    if (topic != "nsPref:changed")
      return;

    var whitelist = pref.getBoolPref(prefName);

    if (whitelist && pref.getBoolPref("privacy.popups.first_popup"))
      pref.setBoolPref("privacy.popups.first_popup", false);

    var count = 0;
    var hosts = [];
    var permissionManager = Components.classes["@mozilla.org/permissionmanager;1"]
                                      .getService(Components.interfaces.nsIPermissionManager);
    var enumerator = permissionManager.enumerator;
    var capability = whitelist ? nsIPermissionManager.DENY_ACTION : nsIPermissionManager.ALLOW_ACTION;

    while (enumerator.hasMoreElements()) {
      var nextPermission = enumerator.getNext();
      nextPermission = nextPermission.QueryInterface(Components.interfaces.nsIPermission);
      if (nextPermission.type == "popup") {
        if (nextPermission.capability == capability)
          hosts[nextPermission.host] = nextPermission.host;
      }
    }
    var popupIcon = document.getElementById("popupIcon");
    var browsers = getBrowser().browsers;
    
    for (var i = 0; i < browsers.length; i++) {
      if (!whitelist && browsers[i].popupDomain in hosts)
        break;
      browsers[i].popupDomain = null;
      popupIcon.hidden = true;
    }
  }
};
////////////////////////////////////////////////////////////////////////////////
function multiZillaInit()
{
  var doInitialize = true;

  if ('arguments' in window && window.arguments.length == 2 && window.arguments[1].indexOf("turbo=yes") != -1)
    doInit = false;

  if (doInitialize) {
    addPrefListener(gTabMinWidthPrefListener);
    addPrefListener(gTabWidthPrefListener);
    addPrefListener(gTabLEDstatePrefListener);
    addPrefListener(gTabLabelPrefListener);
    addPrefListener(gTabLabelCropPrefListener);
    addPrefListener(gTabSwitchPrefListener);
    addPrefListener(gTBButtonPrefListener);
    addPrefListener(gTabCanClosePrefListener);
    addPrefListener(gbToolboxVisibilityPrefListener);
    addPrefListener(gOrdinancePrefListener);
    addPrefListener(gtabFocusOrderPrefListener);
    addPrefListener(gtabbarNewButtonPrefListener);
    addPrefListener(gtabbarCloseButtonPrefListener);
    addPrefListener(gTabSiteIconsPrefListener);
    addPrefListener(gTabFaviconsPrefListener);
    addPrefListener(gTabAdvancedOpenTabPrefListener);
    addPrefListener(gTabSessionStartupPrefListener);
    addPrefListener(gTabMetaRefreshPrefListener);
  }
  initPreferences();
  initializeRDFStorageFile();

  if (!pref) { // Make sure 'pref' is initialized (not the case in Netscape 7)
    var prefService = Components.classes["@mozilla.org/preferences-service;1"]
                                .getService(Components.interfaces.nsIPrefService);
    var pref = prefService.getBranch(null);
  }
}
////////////////////////////////////////////////////////////////////////////////
// this function sets ID's to the Go Menu and replaces the popup
// I know this is a dirty way to do it, but we need to ;)
////////////////////////////////////////////////////////////////////////////////
function addGoMenuIDs()
{
  var mainMenu = document.getElementById("main-menubar");
  var goMenu = mainMenu.childNodes[3];
  goMenu.setAttribute("id", "menu_Go");
  goMenu.firstChild.setAttribute("id", "menu_GoPopup");
  var goMenuPopup = document.getElementById("newMenu_GoPopup");
  // We can't use replaceChild because that is broken in mozilla!!!
  // goMenu.replaceChild(goMenuPopup, goMenu.firstChild);
  goMenu.insertBefore(goMenuPopup, goMenu.firstChild);
  goMenu.removeChild(goMenu.lastChild);
}
////////////////////////////////////////////////////////////////////////////////
function initBookmarkTemplates()
{
  initializeBookmarksTemplate();
  initializePersonalToolbarTemplate();

  // select modified template for the Bookmarks menu
  var bookmarksMenu = document.getElementById("BookmarksMenu");
  bookmarksMenu.setAttribute("template", "BookmarksMenuTemplate");
  // select modified template for the Bookmarks Button
  var bookmarkButton = document.getElementById("bookmarks-button");
  bookmarkButton.setAttribute("template", "BookmarksButtonTemplate");
  // select modified template for the Bookmarks Overflow Menu
  var bookmarksChevron = document.getElementById("bookmarks-chevron");
  bookmarksChevron.setAttribute("template", "BookmarksButtonTemplate");
  // replace default PT template with our modified version
  var originalPTBox = document.getElementById("bookmarks-ptf");
  var newPTBox = document.getElementById("NEW-NC:PersonalToolbarFolder");
  var clonedTemplate = newPTBox.childNodes[1].cloneNode(true);
  originalPTBox.replaceChild(clonedTemplate, originalPTBox.childNodes[1]);

  if (!/Mac/.test(navigator.platform)) { // Modify Main Bookmarks menu
    // bookmarksMenu.removeAttribute("oncommand"); XXX: see bug 3324
    // bookmarksMenu.setAttribute("onclick", "OpenBookmarkURL(event.target, this.database, event)");
  }
  // cleanup: remove dummy bookmarks menu item
  var dummyBookmarkPopup = document.getElementById("dummyBookmarkPopup");
  dummyBookmarkPopup.removeChild(newPTBox);

  // theme support: added for Mac(OSX)Zilla support
  bookmarkButton.firstChild.setAttribute("class", "bookmarks-menupopup");
  bookmarksChevron.firstChild.setAttribute("class", "bookmarks-menupopup");
}
////////////////////////////////////////////////////////////////////////////////
function initInstalledToolbox()
{
  // GoogleBox support: added for MultiZilla's GoogleBox
  mzToolbox = document.getElementById("MultiZillaToolbar");
  mzToolbox.updateButtons("multizilla.showbutton.");
  var toolboxEnabled = readMyPref("multizilla.integrated.toolbar", "bool", false);
  mzToolbox.mzToolboxSetVisibility(toolboxEnabled);
  var showInternalToolbox = document.getElementById("cmd_viewIntegratedToolBox").getAttribute("checked");

  if (toolboxEnabled) {
    if (showInternalToolbox == "true") {
      document.getElementById("internalToolBox").removeAttribute("hidden");
      mzToolbox.refreshMultiZillaButtons();
    }
  }
  else {
    setTimeout(toggleOrEnableGoogleBox, 2500);
  }
}
////////////////////////////////////////////////////////////////////////////////
function initOrdinals()
{
  // Initialize ordinals
  var nodeList = document.getAnonymousNodes(getBrowser().mTabContainer);
  var index = /Mac/.test(navigator.platform) ? 1 : 0;

  for (var i = (0+index); i < nodeList.length; i++) {
    nodeList[i].ordinal = i;
    if (nodeList[i].localName == "hbox") {
      nodeList[i].firstChild.ordinal = 0;
      nodeList[i].lastChild.ordinal = 9999999;
    }
  }
  nodeList[nodeList.length-(1+index)].ordinal = 9999999;
  gBrowser.mTabContainer.advanceSelectedTab = advanceSelectedTab;
}
////////////////////////////////////////////////////////////////////////////////
function initEnhancedPopupBlocker()
{
  // Remove default popup pref listener, because it fails to work properly!
  removePrefListener(gPopupPrefListener);
  // Use a 'heavily' modified copy of it
  addPrefListener(mzPopupPrefListener);
  /****
  * We need to get notified of blocked popup windows so we replace the 
  * original eventlistener for 'DOMPopupBlocked' with our (this) event 
  * listener and call 'onPopupBlocked' (original function) when we're done.
  */
  addEventListener("DOMPopupBlocked", mzOnPopupBlocked, false);
  removeEventListener("DOMPopupBlocked", onPopupBlocked, false);
}
////////////////////////////////////////////////////////////////////////////////
function mzGetBundle(aBundleName)
{
  aBundleName = aBundleName ? "chrome://multiviews/locale/" + aBundleName :
                              "chrome://multiviews/locale/multiviews.properties";
  var aBundle = srGetStrBundle(aBundleName) || null;
  return aBundle;
}
////////////////////////////////////////////////////////////////////////////////
function mzGetTextFromBundle(aStr, aParam, aBundleName)
{
  var aBundle = mzGetBundle(aBundleName);
  if (!aBundle) {
    alert("aBundle error!");
    return "error";
  }
  try {
    var text = aParam ? aBundle.formatStringFromName(aStr, aParam, aParam.lenght) :
                        aBundle.GetStringFromName(aStr);
  } catch(ex) {
    alert("Error: '"+aStr+"' not found in "+(aBundleName ? aBundleName : "multiviews.properties"));
  }
  return text;
}
////////////////////////////////////////////////////////////////////////////////
// Added for compatibility with pre 1.5 mozilla builds (20032406 and later)
////////////////////////////////////////////////////////////////////////////////
function checkConfirmCloseWS(aConfirm)
{
  // Give 'tryToClose' (WindowIsClosing()) a chance to veto if it is defined
  if (typeof(window.tryToClose) == "function") {
    gConfirmForID = aConfirm;
    if (aConfirm == 4) { // onclose="checkConfirmCloseWS(4)" in multizillaOverlay.xul
      return WindowIsClosing();
    }
  }
  else {
    return confirmCloseWS(aConfirm);
  }
  return true;
}
////////////////////////////////////////////////////////////////////////////////
function confirmCloseWS(aConfirm)
{
  /* ::::
     1  = File Menu 'Close Window' & Ctrl-Shift-W
     2  = File Menu 'Exit' & Ctrl-Q
     4  = Click on [X] on Window
     8  = JavaScript close()
     16 = no confirmation for single tab
  :::: */
  if (aConfirm == 2) {
    var lastChoice = readMyPref("multizilla.confirm.quit-session", "bool", false);
    if (lastChoice) {
      saveTabSession();
      return true;
    }
  }
  var confirmPref = readMyPref("multizilla.confirm.quit", "int", 11);
  var aHeader = mzGetTextFromBundle("closeSession");
  var aQuestion = mzGetTextFromBundle("closeSessionQuestion");
  var tabs = gBrowser.mPanelContainer.childNodes.length;

  if (tabs == 1 && confirmPref & 16) {
    saveTabSession();
    return true;
  }
  else if (aConfirm == 8 && document.documentElement.hasAttribute("chromehidden")) {
    saveTabSession();
    return true;
  }
  else if (confirmPref && (confirmPref & aConfirm) == aConfirm) {
    var promptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"].
                        getService(Components.interfaces.nsIPromptService);

    if (aConfirm & 1 || aConfirm & 4) {
      aHeader = mzGetTextFromBundle("closeWindow");
      aQuestion = mzGetTextFromBundle("closeWindowQuestion");
    }
    if (promptService.confirm(window, aHeader, aQuestion)) {
      if (aConfirm == 2) { // Close session confirmed, remember this, write pref!
        writePref("bool", "multizilla.confirm.quit-session", true);
      }
      saveTabSession();
      return true;
    }
    else {
      if (aConfirm == 2) { // Close session, clear pref
        writePref("bool", "multizilla.confirm.quit-session", false);
      }
      return false;
    }
  }
  saveTabSession();
  return true;
}

var firstBrowserWindow = null;
////////////////////////////////////////////////////////////////////////////////
function isFirstWindow(aType)
{
  var defaultType = "navigator:browser";

  if (!aType) // default check for browser window
    aType = defaultType;

  var windowManager = Components.classes['@mozilla.org/appshell/window-mediator;1'].getService();
  var windowManagerInterface = windowManager.QueryInterface(Components.interfaces.nsIWindowMediator);
  var topWindow = windowManagerInterface.getMostRecentWindow(aType);
  var enumerator = windowManagerInterface.getEnumerator(aType);
  var firstWindow = enumerator.getNext().QueryInterface(Components.interfaces.nsIDOMWindowInternal);
  var windowType = topWindow.document.documentElement.getAttribute("windowtype")
  // Initialize 'firstBrowserWindow' the first time only!
  if (firstBrowserWindow == null && windowType == defaultType)
    firstBrowserWindow = firstWindow; // this is used in mzWindowInit()

  if (firstWindow != topWindow)
    return false;
  return true;
}
////////////////////////////////////////////////////////////////////////////////
function getNumberOfWindowsByType(aType)
{
  var defaultType = "navigator:browser";

  if (!aType) // default check for browser window
    aType = defaultType;

  var windowCounter = 0;
  var windowManager = Components.classes['@mozilla.org/appshell/window-mediator;1'].getService(Components.interfaces.nsIWindowMediator);
  var enumerator = windowManager.getEnumerator(aType);

  while (enumerator.hasMoreElements()) {
    var aWindow = enumerator.getNext();
    windowCounter++;
  }
  return windowCounter;
}
////////////////////////////////////////////////////////////////////////////////
function isLastWindow(aType)
{
  return (getNumberOfWindowsByType(aType) == 1) ? true : false;
}
////////////////////////////////////////////////////////////////////////////////
function getMostRecentWindow(aType)
{
  var defaultType = "navigator:browser";

  if (!aType) // default check for browser window
    aType = defaultType;

  var windowManager = Components.classes['@mozilla.org/appshell/window-mediator;1'].getService();
  var windowManagerInterface = windowManager.QueryInterface(Components.interfaces.nsIWindowMediator);
  var navigatorWindow = windowManagerInterface.getMostRecentWindow(aType);

  return navigatorWindow;
}
////////////////////////////////////////////////////////////////////////////////
function getMostRecentTabbrowser()
{
  var win = getMostRecentWindow();
  return win ? getMostRecentWindow().getBrowser() : null;
}
////////////////////////////////////////////////////////////////////////////////
// This function modifies the default mozilla startup.
////////////////////////////////////////////////////////////////////////////////
function mzWindowInit()
{
  var newTab = null;
  var externalLinks = readMyPref("multizilla.newtabfor.external-links", "bool", true);
  var nativeApplications = readMyPref("multizilla.newtabfor.native-applications", "bool", true);
  var doCheck = false;
  var url;

  if ('arguments' in window)
    url = window.arguments[0];

  if ((externalLinks || nativeApplications) && 'arguments' in window && window.opener == null) {
    url = window.arguments[0]; // Single file only for now!
    if (url != "" && url != null && url != "about:blank") {
      if (!isFirstWindow() && firstBrowserWindow != null) {
        var tabbrowser = firstBrowserWindow.getBrowser();
        firstBrowserWindow.focus();
        if (readMyPref("multizilla.tabs.blank-tab-first", "bool", true)) {
          newTab = checkForBlankTab(tabbrowser);
          if (newTab != null)
            newTab.loadURI(url);
        }
        if (newTab == null) { // Add new tab with URL. Pref not set or no blank tab available
          newTab = tabbrowser.addTab(url);
          tabbrowser.selectedTab = newTab; // Don't we need a pref setting for this?
        }
        window.document.documentElement.setAttribute("onunload", "");
        window.close();
        return false;
      }
      // This is, well at least it should be, the first navigator:browser window. 
      else
        doCheck = true;
    }
  }
  if (doCheck || isFirstWindow()) {
    // Sanity check (mozilla crashed before writing our pref setting?).
    mzResetConfirmationPref();
    // Do we need to restore a previously saved browser session?
    if (readMyPref("multizilla.tabs.startgroup", "bool", true)) {
      var startup = pref.getIntPref("browser.startup.page");
      // Browser sessions will only be restored in limited cases
      if (startup == 0 && url == "about:blank")
        getBrowser().mRestoreBrowserSession = true;
      else if (startup == 1 && url == getHomePage())
        getBrowser().mRestoreBrowserSession = true;
      else if (startup == 2) {
        var history = Components.classes["@mozilla.org/browser/global-history;1"].getService(Components.interfaces.nsIBrowserHistory);
        if (url == history.lastPageVisited)
          getBrowser().mRestoreBrowserSession = true;
      }
    }
  }
  Startup();
  if (!getBrowser().mInit)
    getBrowser().enterTabbedMode();
  return true;
}
////////////////////////////////////////////////////////////////////////////////
// Replacement for mozilla's close window prompt in navigator.js
////////////////////////////////////////////////////////////////////////////////
function WindowIsClosing()
{
  var shouldClose = true;
  var browser = getBrowser();

  if (!browser.mChromeHidden) {
    var confirmForID = (gConfirmForID != -1) ? gConfirmForID : 2; // -1 is Composer, Mail, Addressbook or the JS Console
    shouldClose = confirmCloseWS(confirmForID);

    if (shouldClose && isLastWindow()) {
      mzResetConfirmationPref();
      mzCleanupOnExit();
    }
  }
  return shouldClose;
}
////////////////////////////////////////////////////////////////////////////////
//
////////////////////////////////////////////////////////////////////////////////
function mzCleanupOnExit()
{
  var shouldConfirm = readMyPref("multizilla.privacy.clear-on-exit.confirm", "bool", true);

  if (shouldConfirm) {
    var prefIdentifier = ["diskcache", "locationbar", "history", "cookies", "downloads", "formdata", "passwords"];
    var list = "";
    // Initialize list based on pref values
    for (i in prefIdentifier) {
      var prefValue = readMyPref("multizilla.privacy.clear-on-exit." + prefIdentifier[i], "bool", false);
      if (prefValue)
        list = list + prefIdentifier[i] + ", ";
    }
    list = list.replace(/\, $/, ''); // remove last ", "
    // Do we have anything to remove?
    if (list.length) {
      var checkboxMsg = mzGetTextFromBundle("clearOnExitDontAskAgain");
      var title = mzGetTextFromBundle("clearOnExitTitle");
      var text = mzGetTextFromBundle("clearOnExitText");
      text = text + list; // Add list 
      var checkbox = {value:0};
      var promptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"].getService(Components.interfaces.nsIPromptService);
      var result = promptService.confirmEx(window, title, text, 
                                          (promptService.BUTTON_POS_0 * promptService.BUTTON_TITLE_YES) +
                                          (promptService.BUTTON_POS_1 * promptService.BUTTON_TITLE_NO),
                                           null, null, null, null, checkbox);

      if (result == 0) {
        var newPrefIdentifier = list.split(", ");
        for (i in newPrefIdentifier) {
          // alert("identifier :>" +newPrefIdentifier[i]+"<:");
          if (newPrefIdentifier[i] == prefIdentifier[0])
            clearCache("ALL");
          if (newPrefIdentifier[i] == prefIdentifier[1])
            clearUrlBarHistory()
          if (newPrefIdentifier[i] == prefIdentifier[2])
            clearGlobalHistory();
          if (newPrefIdentifier[i] == prefIdentifier[3])
            mzClearAllCookies();
          if (newPrefIdentifier[i] == prefIdentifier[4])
            mzClearAllDownloads();
          if (newPrefIdentifier[i] == prefIdentifier[5])
            mzClearAllFormData();
          if (newPrefIdentifier[i] == prefIdentifier[6])
            mzClearAllPasswords();
        }
      }
    }
  }        
}
////////////////////////////////////////////////////////////////////////////////
// Called from function 'WindowIsClosing()' and function 'mzWindowInit()'.
////////////////////////////////////////////////////////////////////////////////
function mzResetConfirmationPref()
{
  /***
    * This pref setting will be set to 'true' in function confirmCloseWS()
    * after the user confirms to close the this session. Other open windows 
    * will check this pref setting, to enable a more relaxed/quiet browser 
    * exit, with only a single confirmation asked for close session!
    */
  writePref("bool", "multizilla.confirm.quit-session", false);
}